import 'dart:async';
import 'package:flutter/services.dart';
import 'package:interactive_webview/interactive_webview.dart';
import 'package:uuid/uuid.dart';
import 'package:sentry/sentry.dart';
import 'message_broker.dart';
import 'dart:convert';
import 'dto.dart';
import 'player/tivio_player_wrapper.dart';

class Bridge {
  static final Bridge _instance = Bridge._internal();
  final _uuid = Uuid();
  final _webView = InteractiveWebView();
  final _jsonEncoder = JsonEncoder();
  final _dataMessageBroker = DataMessageBroker();
  final _callbackMessageBroker = CallbackMessageBroker();
  Completer _bridgeReady;

  factory Bridge() {
    return _instance;
  }

  Bridge._internal();

  loadBridgeJs() {
    PlatformAssetBundle loader = PlatformAssetBundle();
    return loader.loadString('packages/tivio_sdk/assets/bridge.build.js');
  }

  Future<void> init(String appId, String language) async {
    if (_bridgeReady != null) {
      return _bridgeReady.future;
    }

    try {
      _bridgeReady = Completer();
      String bundle = await loadBridgeJs();

      _webView.didReceiveMessage.listen((event) {
        switch (event.data['type']) {
          case 'data':
            _dataMessageBroker.publish(event.data['messageId'], event.data['data']);
            break;
          case 'callback':
            _callbackMessageBroker.publish(event.data['messageId'], event.data['data']);
            break;
        }
      });

      _webView.stateChanged.listen((event) {
        if (event.type == WebViewState.didFinish) {
          callFunction('init', [appId, language], (payload) => _bridgeReady.complete());
        }
      });

      _webView.loadHTML('''
<!DOCTYPE html>
<html>
  <head>
    <meta charset="UTF-8">
    <script>
      $bundle
    </script>
  </head>
  <body>
  </body>
</html>
''', baseUrl: 'http://tiv.io/');

      return _bridgeReady.future;
    } catch (exception, stackTrace) {
      await Sentry.captureException(
        exception,
        stackTrace: stackTrace,
      );
      throw exception;
    }
  }

  setLanguage(String language) {
    return callFunction('setLanguage', [language], (payload) => null);
  }

  getPlayer(String id) {
    return callFunction<TivioPlayerWrapper>('getPlayer', [id], (payload) => TivioPlayerWrapper(id, this));
  }

  playerRegister(String id) {
    return callFunction('playerRegister', [id], (payload) => null);
  }

  playerOnSourceChanged(String id, data) {
    return callFunction('playerOnSourceChanged', [id, _jsonEncoder.convert(data)], (payload) => null);
  }

  playerPlay(String id) {
    return callFunction('playerPlay', [id], (payload) => {});
  }

  playerPause(String id) {
    return callFunction('playerPause', [id], (payload) => {});
  }

  playerSeekTo(String id, num timestampMs) {
    return callFunction('playerSeekTo', [id, timestampMs], (payload) => {});
  }

  playerOnTimeChanged(String id, num timestampMs) {
    return callFunction('playerOnTimeChanged', [id, timestampMs], (payload) => null);
  }

  playerOnStateChanged(String id, String state) {
    return callFunction('playerOnStateChanged', [id, state], (payload) => null);
  }

  getWidgetById(String widgetId) {
    return callFunction<TivioDataWidget>(
      'getWidgetById',
      [widgetId],
      (payload) => new TivioDataWidget(
        payload['widgetId'],
        payload['name'],
        this,
      ),
    );
  }

  getChannelById(String channelId) {
    return callFunction<TivioDataChannel>('getChannelById', [channelId], (payload) {
      print('channel by id');
      print(payload);
      return TivioDataChannel(
        payload['id'],
        payload['name'],
        null,
        null,
        this,
      );
    });
  }

  getSectionById(String sectionId) {
    return callFunction<TivioDataSection>(
      'getSectionById',
      [sectionId],
      (payload) => TivioDataSection(
        payload['id'],
        payload['name'],
        this,
      ),
    );
  }

  getVideoById(String videoId) {
    return callFunction<TivioDataVideo>(
      'getVideoById',
      [videoId],
      (record) => new TivioDataVideo(
        record['id'],
        record['name'],
        Duration(milliseconds: record['duration'] ?? 0),
        (record['created'] != null) ? DateTime.parse(record['created']) : null,
        record['url'] ?? '',
        record['cover'] ?? '',
        record['isPlaylist'] ?? false,
        [],
        record['monetization'],
      ),
    );
  }

  widgetGetRecentVideos(String widgetId) {
    return callFunction<List<TivioDataVideo>>('widgetGetRecentVideos', [widgetId], (payload) {
      List<TivioDataVideo> videos = [];
      payload.forEach((record) {
        videos.add(TivioDataVideo(
          record['id'],
          record['name'],
          Duration(milliseconds: record['duration'] ?? 0),
          (record['created'] != null) ? DateTime.parse(record['created']) : null,
          record['url'] ?? '',
          record['cover'] ?? '',
          record['isPlaylist'] ?? false,
          [],
          record['monetization'],
        ));
      });
      return videos;
    });
  }

  channelGetRecentVideos(String channelId) {
    return callFunction<List<TivioDataVideo>>('channelGetRecentVideos', [channelId], (payload) {
      List<TivioDataVideo> videos = [];
      payload.forEach((record) {
        videos.add(TivioDataVideo(
          record['id'],
          record['name'],
          Duration(milliseconds: record['duration'] ?? 0),
          (record['created'] != null) ? DateTime.parse(record['created']) : null,
          record['url'] ?? '',
          record['cover'] ?? '',
          record['isPlaylist'] ?? false,
          [],
          record['monetization'],
        ));
      });
      return videos;
    });
  }

  searchChannels(String query, int limit) {
    return getPaginatedData<TivioDataChannel>(
      'searchChannels',
      [query, limit],
      (record) => new TivioDataChannel(
        record['id'],
        record['name'],
        null,
        null,
        this,
      ),
    );
  }

  searchVideos(String query, int limit) {
    return getPaginatedData<TivioDataVideo>(
      'searchVideos',
      [query, limit],
      (record) => new TivioDataVideo(
        record['id'],
        record['name'],
        Duration(milliseconds: record['duration'] ?? 0),
        (record['created'] != null) ? DateTime.parse(record['created']) : null,
        record['url'] ?? '',
        record['cover'] ?? '',
        record['isPlaylist'] ?? false,
        [],
        record['monetization'],
      ),
    );
  }

  getPaginatedData<T>(String functionName, List<dynamic> functionArgs, T Function(dynamic record) createDto) {
    return callFunction<TivioDataPagination<T>>(functionName, functionArgs, (payload) {
      var records = payload['records'].map<T>((record) => createDto(record)).toList();

      return new TivioDataPagination<T>(records, payload['hasNextPage'] ?? false);
    });
  }

  Future<T> callFunction<T>(String functionName, List<dynamic> functionArgs, T Function(dynamic payload) onComplete) {
    Completer done = new Completer<T>();

    String messageId = _uuid.v4();
    functionArgs.add(messageId);
    String serializedArgs = _jsonEncoder.convert(functionArgs);

    _webView.evalJavascript('''window.TivioBridge.$functionName($serializedArgs);''');

    _dataMessageBroker.listen(messageId, (payload) => done.complete(onComplete(payload)));

    return done.future;
  }

  onPlayerPlay(String id, Function callback) {
    _callbackMessageBroker.listen('player-$id-play', callback);
  }

  onPlayerPause(String id, Function callback) {
    _callbackMessageBroker.listen('player-$id-pause', callback);
  }

  onPlayerChangeSource(String id, Function callback) {
    _callbackMessageBroker.listen('player-$id-set-source', callback);
  }

  onPlayerSeekTo(String id, Function callback) {
    _callbackMessageBroker.listen('player-$id-seek-to', callback);
  }

  subscribeToWidgets(String widgetId, Function(TivioDataWidget) callback) {
    _callbackMessageBroker.listen('watch-widgets', (payload) {
      if (payload['widgetId'] != null && payload['name'] != null) {
        callback(TivioDataWidget(payload['widgetId'], payload['name'], this));
      }
    });

    return callFunction('watchWidgets', [widgetId], (payload) => null);
  }

  unsubscribeToWidgets() {
    _callbackMessageBroker.removeListener('watch-widgets');
  }

  subscribeToChannels(String widgetId, Function(TivioDataChannel) callback) {
    _callbackMessageBroker.listen('watch-channels', (payload) {
      if (payload['name'] != null) {
        callback(TivioDataChannel(payload['id'], payload['name'], payload['logo'], payload['cover'], this));
      }
    });

    return callFunction('watchChannels', [widgetId], (payload) => null);
  }

  unsubscribeToChannels() {
    _callbackMessageBroker.removeListener('watch-channels');
  }

  subscribeToSections(String channelId, Function(TivioDataSection) callback) {
    _callbackMessageBroker.listen('watch-sections', (payload) {
      if (payload['name'] != null) {
        callback(TivioDataSection(payload['id'], payload['name'], this));
      }
    });

    return callFunction('watchSections', [channelId], (payload) => null);
  }

  unsubscribeToSections() {
    _callbackMessageBroker.removeListener('watch-sections');
  }

  subscribeToVideos(String sectionId, Function(TivioDataVideo) callback) {
    _callbackMessageBroker.listen('watch-videos', (payload) {
      if (payload['name'] != null) {
        List<TivioDataPlaylistItem> playlistItems = [];

        for (var playlistItem in payload['chapters']) {
          if (playlistItem['name'] != null && playlistItem['duration'] != null && playlistItem['uri'] != null) {
            playlistItems.add(TivioDataPlaylistItem(
              playlistItem['name'],
              playlistItem['uri'],
              Duration(milliseconds: playlistItem['duration']),
            ));
          }
        }

        callback(TivioDataVideo(
          payload['id'],
          payload['name'],
          Duration(milliseconds: payload['duration'] ?? 0),
          (payload['created'] != null) ? DateTime.parse(payload['created']) : null,
          payload['url'] ?? '',
          payload['cover'] ?? '',
          payload['isPlaylist'] ?? false,
          playlistItems,
          payload['monetization'],
        ));
      }
    });

    return callFunction('watchVideos', [sectionId], (payload) => null);
  }

  unsubscribeToVideos() {
    _callbackMessageBroker.removeListener('watch-videos');
  }
}
