import 'package:meta/meta.dart';
import 'package:flutter/cupertino.dart';
import 'package:better_player/better_player.dart';
import 'package:flutter/material.dart';
import '../tivio.dart';
import '../player/tivio_player_input_source.dart';
import '../player/tivio_player_wrapper.dart';
import '../player/tivio_player_controller.dart';

class TivioPlayer extends StatefulWidget {
  String playerId;
  Tivio tivio;
  TivioPlayerController controller;
  TivioPlayerInputSource source;

  TivioPlayer({
    @required String playerId,
    @required Tivio tivio,
    TivioPlayerController controller,
    TivioPlayerInputSource source,
  }) {
    this.playerId = playerId;
    this.tivio = tivio;
    this.controller = (controller == null) ? TivioPlayerController() : controller;
    this.source = source;
  }

  @override
  _TivioPlayerState createState() => _TivioPlayerState();
}

class _TivioPlayerState extends State<TivioPlayer> {
  bool _isTivioInitialized = false;
  bool _isBetterPlayerInitialized = false;
  TivioPlayerWrapper _tivioPlayerWrapper;
  BetterPlayerController _betterPlayerController;

  @override
  void initState() {
    super.initState();
    synchronousInitState();
  }

  synchronousInitState() async {
    await initTivioPlayerWrapper();
    initTivioPlayerWrapperController();

    if (this.widget.source != null) {
      widget.controller.openAndPlay(widget.source);
    }

    setState(() {
      _isTivioInitialized = true;
    });
  }

  initTivioPlayerWrapper() async {
    _tivioPlayerWrapper = await widget.tivio.getPlayer(widget.playerId);

    _tivioPlayerWrapper.register(TivioPlayerWrapperController(
      play,
      pause,
      seekTo,
      openAndPlay,
    ));
  }

  initTivioPlayerWrapperController() {
    widget.controller.play = _tivioPlayerWrapper.play;
    widget.controller.pause = _tivioPlayerWrapper.pause;
    widget.controller.openAndPlay = _tivioPlayerWrapper.openAndPlay;
    widget.controller.seekTo = _tivioPlayerWrapper.seekTo;
  }

  initBetterPlayer(String uri) {
    var config = BetterPlayerConfiguration(
        aspectRatio: 16 / 9,
        fit: BoxFit.contain,
        autoPlay: false,
        autoDispose: true,
        controlsConfiguration: BetterPlayerControlsConfiguration(
          enableOverflowMenu: false,
          enableSkips: false,
        ),
        eventListener: (event) => handleBetterPlayerEvent(event));

    var source = BetterPlayerDataSource(
      BetterPlayerDataSourceType.network,
      uri,
    );

    _betterPlayerController = BetterPlayerController(config);
    _betterPlayerController.setupDataSource(source);

    setState(() {
      _isBetterPlayerInitialized = true;
    });
  }

  @override
  Widget build(BuildContext context) {
    return AspectRatio(
      aspectRatio: 16 / 9,
      child: (_isTivioInitialized && _isBetterPlayerInitialized)
          ? BetterPlayer(
              controller: _betterPlayerController,
            )
          : Container(
              decoration: BoxDecoration(
                color: Colors.black,
              ),
            ),
    );
  }

  play(payload) {
    _betterPlayerController.play();
  }

  pause(payload) {
    _betterPlayerController.pause();
  }

  openAndPlay(source) {
    if (!_isBetterPlayerInitialized) {
      initBetterPlayer(source.uri);
    }

    _betterPlayerController.setupDataSource(
      BetterPlayerDataSource(
        BetterPlayerDataSourceType.network,
        source.uri,
      ),
    );

    if (source.type == TivioPlayerWrapperInputSourceType.Advertisement) {
      _betterPlayerController.setControlsEnabled(false);
    } else {
      _betterPlayerController.setControlsEnabled(true);
    }

    _betterPlayerController.play();
  }

  seekTo(Duration time) {
    _betterPlayerController.seekTo(time);
  }

  handleBetterPlayerEvent(BetterPlayerEvent event) {
    switch (event.betterPlayerEventType) {
      // TD add buffering state. Currently isn't supported by the betterplayer.
      // file: lib/src/core/better_player_controller.dart, method: _handleVideoEvent
      case BetterPlayerEventType.play:
        _tivioPlayerWrapper.onStateChanged(TivioPlayerState.playing);
        break;
      case BetterPlayerEventType.pause:
        _tivioPlayerWrapper.onStateChanged(TivioPlayerState.paused);
        break;
      case BetterPlayerEventType.progress:
        _tivioPlayerWrapper.onTimeChanged(event.parameters['progress']);
        break;
      case BetterPlayerEventType.seekTo:
        _tivioPlayerWrapper.onTimeChanged(event.parameters['duration']);
        break;
      default:
        break;
    }
  }
}
